@@ -1,13 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import cPickle
-try:
-    import json
-except ImportError:
-    try:
-        from django.utils import simplejson as json
-    except Exception, e:
-        import simplejson as json
+from common import json
 
 try:
     import yaml
@@ -188,7 +182,7 @@ class YAMLFormatter(Formatter):
     __mime__ = "application/x-yaml"
 
     def build(self, value):
-        return yaml.dump(value)
+        return yaml.safe_dump(value)
 
     def kwargs(self, value, action='build'):
         if action == 'build':
@@ -197,7 +191,7 @@ class YAMLFormatter(Formatter):
             return self.parse(value)
 
     def parse(self, value):
-        return yaml.load(value)
+        return yaml.safe_load(value)
 
 formatters.register('json', JSONFormatter)
 formatters.register('jsonp', JSONPFormatter)
@@ -205,4 +199,4 @@ formatters.register('value', ValueFormatter)
 formatters.register('pickle', PickleFormatter)
 formatters.register('xml', XMLFormatter)
 if has_yaml:
-    formatters.register('yaml', YAMLFormatter)
\ No newline at end of file
+    formatters.register('yaml', YAMLFormatter)
