@@ -1,16 +1,9 @@
 # -*- coding: utf-8 -*-
 
-try:
-    import json
-except ImportError, e:
-    try:
-        from django.utils import simplejson as json
-    except Exception, e:
-        import simplejson as json
-
 import re
 import datetime
 from dateutil.parser import parse
+from common import json
 
 __all__ = ('SimpleAPIEncoder', 'SimpleAPIDecoder')
 
