@@ -1,9 +1,11 @@
 # -*- coding: utf-8 -*-
 
 __all__ = ('Route', 'Namespace', 'Feature', 'FeatureContentResponse', 
-           'serialize')
+           'serialize', 'UnformattedResponse', 'RouteMgr')
 
 from route import Route
+from routemgr import RouteMgr
 from namespace import Namespace
 from feature import Feature, FeatureContentResponse
-from serializer import serialize
\ No newline at end of file
+from serializer import serialize
+from response import UnformattedResponse
\ No newline at end of file
