@@ -5,6 +5,7 @@ import hashlib
 import warnings
 
 from simpleapi.message import formatters, Formatter
+from simpleapi.message.common import SAException
 
 try:
     from django.core.cache import cache
@@ -14,7 +15,7 @@ except:
 
 __all__ = ('__features__', 'Feature', 'FeatureException', 'FeatureContentResponse')
 
-class FeatureException(Exception): pass
+class FeatureException(SAException): pass
 class FeatureContentResponse(FeatureException): pass
 class Feature(object):
 
