@@ -1,15 +1,19 @@
 # -*- coding: utf-8 -*-
 
+import warnings
 import tempfile
 import pprint
-import cProfile
-import pstats
-import logging
+try:
+    import cProfile
+    import pstats
+    has_debug = True
+except ImportError:
+    has_debug = False
 
 from response import Response
-from session import Session
 from feature import FeatureContentResponse
 from simpleapi.message import formatters
+from simpleapi.message.common import SAException
 
 try:
     from django.core.exceptions import ObjectDoesNotExist as django_notexist
@@ -25,12 +29,12 @@ except ImportError:
 
 __all__ = ('Request', 'RequestException')
 
-class RequestException(Exception): pass
+class RequestException(SAException): pass
 class Request(object):
 
     def __init__(self, sapi_request, namespace, input_formatter,
                  output_formatter, wrapper, callback, mimetype, restful,
-                 debug):
+                 debug, route, ignore_unused_args):
         self.sapi_request = sapi_request
         self.namespace = namespace
         self.input_formatter = input_formatter
@@ -40,16 +44,18 @@ class Request(object):
         self.mimetype = mimetype
         self.restful = restful
         self.debug = debug
-        self.session = Session()
+        self.route = route
+        self.ignore_unused_args = ignore_unused_args
+        self.session = sapi_request.session
 
-    def run(self, request_items):
+    def process_request(self, request_items):
         # set all required simpleapi arguments
         access_key = request_items.pop('_access_key', None)
         method = request_items.pop('_call', None)
-        
+
         if self.restful:
             method = self.sapi_request.method.lower()
-        
+
         data = request_items.pop('_data', None)
 
         # update session
@@ -58,13 +64,17 @@ class Request(object):
         self.session.callback = self.callback
         self.session.access_key = access_key
 
+        # make all uploaded files available
+        if self.route.is_django():
+            self.session.files = self.sapi_request.FILES
+
         # instantiate namespace
         local_namespace = self.namespace['class'](self)
-        self.session.namespace = {
+        self.session._internal.namespace = {
             'nmap': self.namespace,
             'instance': local_namespace
         }
-        
+
         # check the method
         if not method:
             raise RequestException(u'Method must be provided.')
@@ -86,7 +96,7 @@ class Request(object):
         self.session.function = function
 
         # check allowed HTTP methods
-        if not function['methods']['function'](self.sapi_request.method):
+        if not function['methods']['function'](self.sapi_request.method, function['methods']['allowed_methods']):
             raise RequestException(u'Method not allowed: %s' % self.sapi_request.method)
 
         # if data is set, make sure input formatter is not ValueFormatter
@@ -113,12 +123,16 @@ class Request(object):
 
         # check whether there are more arguments than needed
         if not function['args']['kwargs_allowed']:
-            unsued_arguments = list(set(request_items.keys()) - \
+            unused_arguments = list(set(request_items.keys()) - \
                 set(function['args']['all']))
 
-            if unsued_arguments:
-                raise RequestException(u'Unused arguments: %s' % \
-                ", ".join(unsued_arguments))
+            if unused_arguments:
+                if not self.ignore_unused_args:
+                    raise RequestException(u'Unused arguments: %s' % \
+                    ", ".join(unused_arguments))
+                else:
+                    for key in unused_arguments:
+                        del request_items[key]
 
         # decode incoming variables (only if _data is not set!)
         if not data:
@@ -157,14 +171,18 @@ class Request(object):
         except FeatureContentResponse, e:
             result = e
         else:
+            # call before_request
+            if hasattr(local_namespace, 'before_request'):
+                getattr(local_namespace, 'before_request')(self, self.session)
+            
             # make the call
             try:
                 if self.debug:
                     _, fname = tempfile.mkstemp()
-                    logging.debug(u"Profiling call '%s': %s" % \
+                    self.route.logger.debug(u"Profiling call '%s': %s" % \
                         (method, fname))
 
-                    logging.debug(u"Calling parameters: %s" % \
+                    self.route.logger.debug(u"Calling parameters: %s" % \
                         pprint.pformat(request_items))
 
                     profile = cProfile.Profile()
@@ -172,7 +190,7 @@ class Request(object):
                         **request_items)
                     profile.dump_stats(fname)
                     
-                    logging.debug(u"Loading stats...")
+                    self.route.logger.debug(u"Loading stats...")
                     stats = pstats.Stats(fname)
                     stats.strip_dirs().sort_stats('time', 'calls') \
                         .print_stats(25)
@@ -195,7 +213,6 @@ class Request(object):
                 output_formatter=self.output_formatter,
                 wrapper=self.wrapper,
                 mimetype=self.mimetype,
-                session=self.session,
                 function=function
             )
         else:
