@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import types
 import copy
 import inspect
 import pprint
@@ -7,8 +8,16 @@ import re
 import sys
 import os
 import pdb
-import cProfile
-import pstats
+import warnings
+import logging
+
+try:
+    import cProfile
+    import pstats
+    has_debug = True
+except ImportError:
+    has_debug = False
+
 import urlparse
 import cgi
 from wsgiref.simple_server import make_server
@@ -21,24 +30,33 @@ SIMPLEAPI_DEBUG_LEVEL = os.environ.get('SIMPLEAPI_DEBUG_LEVEL', 'all')
 assert SIMPLEAPI_DEBUG_LEVEL in ['all', 'call'], \
     u'SIMPLEAPI_DEBUG_LEVEL must be one of these: all, call'
 
+if SIMPLEAPI_DEBUG and not has_debug:
+    SIMPLEAPI_DEBUG = False
+    warnings.warn("Debugging disabled since packages pstats/cProfile not found (maybe you have to install it).")
+
 TRIGGERED_METHODS = ['get', 'post', 'put', 'delete']
 FRAMEWORKS = ['flask', 'django', 'appengine', 'dummy', 'standalone', 'wsgi']
 MAX_CONTENT_LENGTH = 1024 * 1024 * 16 # 16 megabytes
 
+restricted_functions = [
+    'before_request',
+    'after_request'
+]
+
 try:
     from google.appengine.ext.webapp import RequestHandler as AE_RequestHandler
     has_appengine = True
 except ImportError:
     has_appengine = False
 
+from simpleapi.message.common import SAException
 from sapirequest import SAPIRequest
 from request import Request, RequestException
-from response import Response, ResponseException
+from response import Response, ResponseMerger, ResponseException
 from namespace import NamespaceException
 from feature import __features__, Feature, FeatureException
 from simpleapi.message import formatters, wrappers
 from utils import glob_list
-import logging
 
 __all__ = ('Route', )
 
@@ -77,30 +95,46 @@ class Route(object):
             return Router(*args, **kwargs)
 
 class StandaloneRequest(object): pass
-class RouterException(Exception): pass
+class RouterException(SAException): pass
 class Router(object):
 
     def __init__(self, *namespaces, **kwargs):
         """Takes at least one namespace. 
         """
+        self.name = kwargs.pop('name', str(id(self)))
+        self.logger = logging.getLogger("simpleapi.%s" % self.name)
         self.nmap = {}
-        self.debug = kwargs.get('debug', False)
-        self.restful = kwargs.get('restful', False)
-        self.framework = kwargs.get('framework', 'django')
-        self.path = re.compile(kwargs.get('path', r'^/'))
-        
+        self.debug = kwargs.pop('debug', False)
+        self.ignore_unused_args = kwargs.pop('ignore_unused_args', False)
+
+        if self.debug and not has_debug:
+            self.debug = False
+            warnings.warn("Debugging disabled since packages pstats/cProfile not found (maybe you have to install it).")
+
+        self.restful = kwargs.pop('restful', False)
+        self.framework = kwargs.pop('framework', 'django')
+        self.path = re.compile(kwargs.pop('path', r'^/'))
+
+        assert len(kwargs) == 0, u'Unknown Route configuration(s) (%s)' % \
+            ", ".join(kwargs.keys())
+
         # make shortcut 
         self._caller = self.__call__
-        
+
         assert self.framework in FRAMEWORKS
         assert (self.debug ^ SIMPLEAPI_DEBUG) or \
             not (self.debug and SIMPLEAPI_DEBUG), \
             u'You can either activate Route-debug or simpleapi-debug, not both.'
 
         if self.debug or SIMPLEAPI_DEBUG:
-            logging.basicConfig(level=logging.DEBUG)
+            self.logger.setLevel(logging.DEBUG)
+            handler = logging.StreamHandler()
+            formatter = logging.Formatter(
+                "%(asctime)s - %(name)s - %(levelname)s - %(message)s")
+            handler.setFormatter(formatter)
+            self.logger.addHandler(handler)
         else:
-            logging.basicConfig(level=logging.WARNING)
+            self.logger.setLevel(logging.WARNING)
         
         if SIMPLEAPI_DEBUG and SIMPLEAPI_DEBUG_LEVEL == 'all':
             self.profile_start()
@@ -176,22 +210,25 @@ class Router(object):
 
     def serve(self, host='', port=5050):
         httpd = make_server(host, port, self.handle_request)
-        logging.info(u"Started serving on port %d..." % port)
+        self.logger.info(u"Started serving on port %d..." % port)
         try:
             httpd.serve_forever()
         except KeyboardInterrupt:
-            logging.info(u"Server stopped.")
+            self.logger.info(u"Server stopped.")
 
     def profile_start(self):
+        assert has_debug
         self.profile = cProfile.Profile()
         self.profile.enable()
     
     def profile_stop(self):
+        assert has_debug
         self.profile.disable()
         self.profile.dump_stats(SIMPLEAPI_DEBUG_FILENAME)
 
     def profile_stats(self):
-        logging.debug(u"Loading stats...")
+        assert has_debug
+        self.logger.debug(u"Loading stats...")
         stats = pstats.Stats(SIMPLEAPI_DEBUG_FILENAME)
         stats.strip_dirs().sort_stats('time', 'calls') \
             .print_stats()
@@ -240,14 +277,25 @@ class Router(object):
         # make sure no version is assigned twice
         assert not self.nmap.has_key(version), u'version is assigned twice'
 
+        allowed_functions = []
+
+        # check for introspection allowed
+        if getattr(namespace, '__introspection__', False):
+            allowed_functions.append('introspect')
+
         # determine public and published functions
-        functions = filter(lambda item: '__' not in item[0] and
-            getattr(item[1], 'published', False) == True,
+        functions = filter(lambda item: '__' not in item[0] and item[0] not in 
+            restricted_functions and ((getattr(item[1], 'published', False) ==
+            True) or item[0] in allowed_functions),
             inspect.getmembers(namespace))
 
         # determine arguments of each function
         functions = dict(functions)
         for function_name, function_method in functions.iteritems():
+            # check for reserved function names
+            assert function_name not in ['error', '__init__', 'get_name'],\
+                u'Name %s is reserved.' % function_name
+            
             # ArgSpec(args=['self', 'a', 'b'], varargs=None, keywords=None, defaults=None)
             raw_args = inspect.getargspec(function_method)
 
@@ -304,10 +352,10 @@ class Router(object):
             if hasattr(function_method, 'methods'):
                 allowed_methods = function_method.methods
                 assert isinstance(allowed_methods, (list, tuple))
-                method_function = lambda method: method in allowed_methods
+                method_function = lambda method, methods: method in methods
             else:
                 allowed_methods = None
-                method_function = lambda method: True
+                method_function = lambda method, methods: True
 
             # determine format
             format = getattr(function_method, 'format', lambda val: val)
@@ -422,12 +470,11 @@ class Router(object):
 
     def __call__(self, http_request=None, **urlparameters):
         sapi_request = SAPIRequest(self, http_request)
-
         request_items = dict(sapi_request.REQUEST.items())
         request_items.update(urlparameters)
 
         if SIMPLEAPI_DEBUG and SIMPLEAPI_DEBUG_LEVEL == 'call':
-            logging.info(pprint.pformat(request_items))
+            self.logger.info(pprint.pformat(request_items))
             self.profile_start()
 
         version = request_items.pop('_version', 'default')
@@ -498,38 +545,72 @@ class Router(object):
                 callback=callback,
                 mimetype=mimetype,
                 restful=self.restful,
-                debug=self.debug
+                debug=self.debug,
+                route=self,
+                ignore_unused_args=self.ignore_unused_args,
             )
-            response = request.run(request_items)
-            http_response = response.build()
+
+            # map request items to the correct names
+            wi = wrapper_instance(sapi_request=sapi_request)
+            request_items = wi._parse(request_items)
+            if not isinstance(request_items,
+                (list, tuple, types.GeneratorType)):
+                request_items = [request_items, ]
+
+            responses = []
+            for request_item in request_items:
+                # clear session (except _internal)
+                sapi_request.session.clear()
+
+                # process request
+                try:
+                    responses.append(request.process_request(request_item))
+                except (NamespaceException, RequestException, \
+                        ResponseException, RouterException, FeatureException),e:
+                    response = Response(
+                        sapi_request,
+                        errors=e.message,
+                        output_formatter=output_formatter_instance,
+                        wrapper=wrapper_instance,
+                        mimetype=mimetype
+                    )
+                    responses.append(response)
+
+            rm = ResponseMerger(
+                sapi_request=sapi_request,
+                responses=responses,
+            )
+            http_response = rm.build()
         except Exception, e:
-            if isinstance(e, (NamespaceException, RequestException,
-               ResponseException, RouterException, FeatureException)):
-                err_msg = unicode(e)
+            if isinstance(e, (NamespaceException, RequestException, \
+                              ResponseException, RouterException, \
+                              FeatureException)):
+                err_msg = repr(e)
             else:
                 err_msg = u'An internal error occurred during your request.'
-                trace = inspect.trace()
-                msgs = []
-                msgs.append('')
-                msgs.append(u"******* Exception raised *******")
-                msgs.append(u'Exception type: %s' % type(e))
-                msgs.append(u'Exception msg: %s' % e)
-                msgs.append('')
-                msgs.append(u'------- Traceback follows -------')
-                for idx, item in enumerate(trace):
-                    msgs.append(u"(%s)\t%s:%s (%s)" % 
-                        (idx+1, item[3], item[2], item[1]))
-                    if item[4]:
-                        for line in item[4]:
-                            msgs.append(u"\t\t%s" % line.strip())
-                    msgs.append('') # blank line
-                msgs.append('     -- End of traceback --     ')
-                msgs.append('')
-                logging.error("\n".join(msgs))
+            
+            trace = inspect.trace()
+            msgs = []
+            msgs.append('')
+            msgs.append(u"******* Exception raised *******")
+            msgs.append(u'Exception type: %s' % type(e))
+            msgs.append(u'Exception msg: %s' % repr(e))
+            msgs.append('')
+            msgs.append(u'------- Traceback follows -------')
+            for idx, item in enumerate(trace):
+                msgs.append(u"(%s)\t%s:%s (%s)" % 
+                    (idx+1, item[3], item[2], item[1]))
+                if item[4]:
+                    for line in item[4]:
+                        msgs.append(u"\t\t%s" % line.strip())
+                msgs.append('') # blank line
+            msgs.append('     -- End of traceback --     ')
+            msgs.append('')
+            self.logger.error("\n".join(msgs))
 
-                if self.debug:
-                    e, m, tb = sys.exc_info()
-                    pdb.post_mortem(tb)
+            if self.debug:
+                e, m, tb = sys.exc_info()
+                pdb.post_mortem(tb)
 
             response = Response(
                 sapi_request,
