@@ -6,9 +6,14 @@ try:
 except ImportError:
     has_flask = False
 
+from session import Session
+
+__all__ = ('SAPIRequest', )
+
 class SAPIRequest(object):
     
     def __init__(self, route, request=None):
+        self.session = Session()
         self.route = route
 
         if not request:
@@ -37,7 +42,7 @@ class SAPIRequest(object):
     @property
     def POST(self):
         if self.route.is_flask():
-            return self.request.args
+            return self.request.form or self.request.args
         elif self.route.is_django():
             return self.request.POST
         elif self.route.is_appengine():
@@ -49,7 +54,7 @@ class SAPIRequest(object):
     @property
     def REQUEST(self):
         if self.route.is_flask():
-            return self.request.args
+            return self.request.form or self.request.args
         elif self.route.is_django():
             return self.request.REQUEST
         elif self.route.is_appengine():
@@ -59,6 +64,16 @@ class SAPIRequest(object):
             return self.request.data
         raise NotImplementedError
 
+    @property
+    def FILES(self):
+        if self.route.is_django():
+            return self.request.FILES
+        
+        # TODO XXX
+        # FILES access to other frameworks? flask? GAE?
+        
+        raise NotImplementedError
+
     @property
     def META(self):
         if self.route.is_flask():
@@ -88,4 +103,15 @@ class SAPIRequest(object):
             return self.request.method
         elif self.route.is_dummy() or self.route.is_standalone():
             return self.request.method
-        raise NotImplementedError
\ No newline at end of file
+        raise NotImplementedError
+    
+    @property
+    def path_info(self):
+        if self.route.is_flask() or self.route.is_django():
+            return self.META.get('PATH_INFO')
+        # TODO XXX FIXME:
+        #elif self.route.is_appengine():
+        #    return self.request.path_info
+        #elif self.route.is_dummy() or self.route.is_standalone():
+        #    return self.request.path_info
+        raise NotImplementedError
